/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief  XMEGA SPI interrupt-driven driver example source.
 *
 *      This file contains an example application that demonstrates the SPI
 *      interrupt-driven driver.
 *
 * \par Application note:
 *      AVR1309: Using the XMEGA SPI
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 763 $
 * $Date: 2007-11-06 14:51:56 +0100 (ti, 06 nov 2007) $  \n
 *
 * Copyright (c) 2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "avr_compiler.h"
#include "spi_driver_2wire.h"

/*! \brief Number of test data bytes. */
#define NUM_BYTES   2




/* Global variables. */

/*! \brief SPI master on PORT C. */
SPI_Master_t spiMasterC;

/*! \brief Data packet. */
SPI_DataPacket_t dataPacket;

/*! \brief Test data to send. */
//uint8_t sendData[NUM_BYTES+1 ] = { NUM_BYTES, 0x11, 0x33 };
uint8_t sendData[NUM_BYTES] = { 0x11, 0x33 };



/*! \brief Buffer for test data reception. */
uint8_t receivedData[NUM_BYTES + 1];

//********************** INIT ******************************************

void osc_init(void){
//++++++++++++++++++++ externer Quarz 16Mhz auf 32Mhz(intern) +++++++++++++++++++++++++++++++++++++++++
	
	OSC.XOSCCTRL|=  OSC_XOSCSEL_XTAL_16KCLK_gc| OSC_FRQRANGE_12TO16_gc ;
	OSC.PLLCTRL= OSC_PLLSRC_XOSC_gc | (1<<OSC_PLLFAC1_bp);  // PLL auf externen Osc einstellen & *2

	OSC.CTRL |= (1<<OSC_PLLEN_bp) | (1<<OSC_XOSCEN_bp);// PLL & externen OSC enable

	while(!(OSC.STATUS & OSC_PLLRDY_bm));

	CCP = CCP_IOREG_gc; //IOProtect

	CLK.CTRL |= CLK_SCLKSEL_PLL_gc; // PLL als Clock auswhlen
}	

void port_init(void){
	
	/* Initialize PORTE for output */
//		PORTE.DIRSET = 0xFF;
	
	/* Init SS pin as output with wired AND and pull-up. */
		PORTC.DIRSET = PIN4_bm;
		PORTC.PIN4CTRL = PORT_OPC_WIREDANDPULL_gc;	

	/* Set SS output to high. (No slave addressed). */
		PORTC.OUTSET = PIN4_bm;					

}

void ir_init(void){
	/* Enable low and medium level interrupts in the interrupt controller. */
		PMIC.CTRL = PMIC_LOLVLEN_bm ;
		sei();

}


void spi_master_init(void){
	/* Initialize SPI master on port C. */
	SPI_MasterInit(&spiMasterC,
	               &SPIC,
	               &PORTC,
				   false,
	               SPI_MODE_3_gc,
	               SPI_INTLVL_LO_gc,
	               false, //if true SPI_PRESCALER_DIVx_gc *2
 	               SPI_PRESCALER_DIV4_gc);

}




//**************************** MAIN *************************************
int main( void )
{
	//externen Quarz auf 32MHz einstellen
	osc_init();

	port_init();

	spi_master_init();

	ir_init();

	delay_us(1000000);                                    
		/* Create data packet (SS to slave by PC4) */
	SPI_MasterCreateDataPacket(&dataPacket,
	                           sendData,
	                           receivedData,
	                           NUM_BYTES /*Bytes to Sende*/,
							   NUM_BYTES/*Bytes to Recieve*/,
	                       	   &PORTC,
	                           PIN4_bm/*SS*/);

	uint8_t status;
	
		do {
			status = SPI_MasterInterruptTransceivePacket(&spiMasterC, &dataPacket);
		} while (status != SPI_OK);

		/* Wait for transmission to complete. */
		while (dataPacket.complete == false) {
			nop();
		}
		
		
/*		
		do {
			status = SPI_MasterInterruptTransceivePacket(&spiMasterC, &dataPacket);
		} while (status != SPI_OK);
*/
		/* Wait for transmission to complete. */
/*		while (dataPacket.complete == false) {
			nop();
		}
*/

	while(1){
	}

}

// ******************************* ISR ***************************************
ISR(SPIC_INT_vect)
{
		SPI_MasterInterruptHandler(&spiMasterC);



}
